import logging
import os
from pathlib import Path

from dotenv import find_dotenv, load_dotenv
from imap_tools import AND, MailBox

logger = logging.getLogger(__name__)


def extract_email(resource, **kwargs):
    """
    Extract the most recent email matching a subject and save its attachments
    to the given resource path.
    """

    try:
        resource_path = Path(resource.path)
        dpetl_extract = resource.custom.get('dpetl_extract', {})
        criteria = dpetl_extract.get('criteria', {})
        # Only set subject to resource.name if this key doesn't exists
        # on dpetl_extract.criteria custom property
        # Could be dangerous because it changes criteria dict in place
        criteria.setdefault('subject', resource.name)

        resource_path.parent.mkdir(parents=True, exist_ok=True)
        load_dotenv(find_dotenv(usecwd=True))
        email_user = os.environ.get('EMAIL_USER')
        email_pwd = os.environ.get('EMAIL_PWD')
        email_smtp = os.environ.get('EMAIL_SMTP')
        email_box = dpetl_extract.get('mailbox', 'INBOX')

        if not all([email_user, email_pwd, email_smtp]):
            logger.error(
                ('Missing one of the required email environment variables:'
                'email_user, email_pwd or email_smtp.')
            )
            return

        if 'mailbox' not in dpetl_extract:
            logger.debug("Using default mailbox 'INBOX'.")

        logger.debug(
            'Connecting to mailbox.',
            extra={
                'smtp_server': email_smtp,
                'mailbox': email_box,
                'user': email_user,
            },
        )

        with MailBox(email_smtp).login(email_user, email_pwd) as mailbox:
            logger.info('Connected to mailbox successfully.')

            mailbox.folder.set(email_box)
            logger.info(
                'Mailbox folder selected.', extra={'folder': email_box}
            )
            search_query = AND(**criteria)
            logger.debug(
                'Searching emails.', extra={'criteria': str(criteria)}
            )

            msgs = list(mailbox.fetch(search_query, limit=1, reverse=True))

            if not msgs:
                logger.warning(
                    'No email found matching the criteria.',
                    extra={'criteria': str(criteria)},
                )
                return
            elif len(msgs) > 1:
                # No neet to stop the code, just inform
                logger.warning(
                    'More than one email was found matching the criteria.',
                    extra={'criteria': str(criteria)},
                )

            msg = msgs[0]

            logger.info(
                'Email found.',
                extra={
                    'subject': msg.subject,
                    'date': str(msg.date),
                    'from': msg.from_,
                },
            )

            if not msg.attachments:
                logger.warning(
                    'Email found but no attachments presented.',
                    extra={'subject': msg.subject},
                )
                return
            elif len(msg.attachments) > 1:
                # No neet to stop the code, just inform
                logger.warning(
                    'More than one attachment file found.',
                    extra={'criteria': str(criteria)},
                )

            for index, att in enumerate(msg.attachments):
                logger.debug(
                    'Processing attachment.',
                    extra={
                        'filename': att.filename,
                        'size_bytes': len(att.payload),
                    },
                )

                path = (
                    resource_path
                    if index == 0
                    else resource_path.with_name(
                        f'{resource_path.name}_{index}'
                    )
                )
                # Do not need to write in chuncks because mail fails are small
                with open(path, 'wb') as f:
                    f.write(att.payload)

                logger.info(
                    'Attachment saved successfully.',
                    extra={
                        'filename': att.filename,
                        'saved_to': str(path),
                    },
                )

        logger.info('Email extraction completed successfully.')

    except Exception:
        logger.exception('Unexpected error during email extraction.')



import argparse

from .extract.cli import create_extract_subcommands


def build_parser():
    parser = argparse.ArgumentParser(
        prog='etl',
        description='ETL Command Line Interface',
    )

    parser.add_argument(
        '--descriptor',
        '-d',
        default='datapackage.yaml',
        help='Path to a datapackage.yaml file (default: datapackage.yaml).',
    )

    subparsers = parser.add_subparsers(
        title='commands',
        dest='command',
        required=True,
    )

    create_extract_subcommands(subparsers)

    return parser


def main():
    parser = build_parser()
    args = parser.parse_args()
    args.func(args)



import requests


def extract_api(resource, **kwargs):
    try:
        sources = resource.sources
        sources = [
            source for source in sources if source.get('method') is not None
        ]
        source = sources[0]
        if source:
            func = getattr(requests, source['method'])
            response = func(
                source['path'],
                timeout=source.get('timeout', 30),
                params=source.get('params', {}),
                headers=source.get('headers', {}),
                stream=source.get('stream', False),
            )
            response.raise_for_status()

        with open(resource.path, 'wb') as f:
            f.write(response.content)

    except requests.exceptions.RequestException as e:
        print(f'Error downloading file: {e}')
        raise
    except IOError as e:
        print(f'Error saving file: {e}')
        raise



from dpetl.helpers import resources_iteration


def create_extract_subcommands(subparsers):

    parser = subparsers.add_parser(
        'extract', help='Simplified some ETL extract operations.'
    )

    parser.set_defaults(func=handle_command)

    return parser


def handle_command(args):

    resources_iteration(**vars(args))



import logging

from frictionless import Package

from .extract import api, email

logger = logging.getLogger(__name__)


def resources_iteration(**kwargs):
    """
    Iterate on resources from a package descriptor or a package object
    and apply a function to each resource.
    """
    descriptor = kwargs.get('descriptor')
    if descriptor:
        package = Package(descriptor)

    for resource in package.resources:
        mode = resource.custom.get('dpetl_extract', {}).get('mode')

        if not mode:
            logger.error(
                ('Missing required dpetl_extract.mode custom property'
                'at the resource level.'),
                extra={
                    'resource': resource.name,
                },
            )
            return

        logger.info(
            'Starting resource extraction.',
            extra={
                'resource': resource.name,
                'mode': mode,
            },
        )

        if mode == 'email':
            email.extract_email(resource, **kwargs)
        elif mode == 'api':
            api.extract_api(resource, **kwargs)
